@@ -1,39 +1,41 @@
 /*
  *  export_mov.c
  *
- *  Copyright (C) Thomas streich - June 2001
- *  Copyright (C) (2002) Christian Vogelgsang 
+ *  Copyright (C) Thomas Oestreich - June 2001
+ *  Copyright (C) (2002) Christian Vogelgsang
  *  <Vogelgsang@informatik.uni-erlangen.de> (extension for all codecs supported
  *  by quicktime4linux
  *  <stefanscheffler@gmx.net> 2004 fixes & features
  *  Copyright (C) ken@hero.com 2001 (initial module author)
  *
  *  This file is part of transcode, a video stream processing tool
- *      
+ *
  *  transcode is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  transcode is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU General Public License
  *  along with GNU Make; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include "import/magic.h"
-#include "transcode.h"
-#include "vid_aux.h"
-
 #define MOD_NAME    "export_mov.so"
 #define MOD_VERSION "v0.1.2 (2004-01-19)"
 #define MOD_CODEC   "(video) * | (audio) *"
+
+#include "transcode.h"
+#include "import/magic.h"
+#include "encoder.h"
+#include "libtcvideo/tcvideo.h"
+#include <stdio.h>
+#include <stdlib.h>
+
 /* verbose flag */
 static int verbose_flag=TC_QUIET;
 
@@ -65,23 +67,23 @@ static quicktime_t *qtfile = NULL;
 /* row pointer for source frames */
 static unsigned char** row_ptr = NULL;
 
-/* temporary buffer*/
-static unsigned char *tmp_buf = NULL;
+/* temporary buffer */
+static uint8_t *tmp_buf;
 
 /* toggle for raw frame export */
 static int rawVideo = 0;
 static int rawAudio = 0;
 
+/* colormodels */
+static ImageFormat tc_cm = 0;
+static int qt_cm = 0;
+
 /* store frame dimension */
-static int w = 0,h = 0;
+static int w = 0, h = 0;
 
 /* audio channels */
 static int channels = 0;
 
-/* colormodels */
-static int tc_cm = 0;
-static int qt_cm = 0;
-
 /* sample size */
 static int bits = 0;
 
@@ -95,6 +97,8 @@ struct qt_codec_list {
     char *comments;
 };
 
+static TCVHandle tcvhandle = 0;
+
 /* special paramters not retrievable from lqt */
 struct qt_codec_list qt_param_list[] = {
   {"", "",  ""},
@@ -104,26 +108,27 @@ struct qt_codec_list qt_param_list[] = {
   {"info", "",  "Info string (no '=' or ',' allowed) "},
   {NULL, NULL, NULL}};
 
+#ifdef LIBQUICKTIME_000904
 /* from libquicktime */
-int tc_quicktime_get_timescale(double frame_rate)
+static int tc_quicktime_get_timescale(double frame_rate)
 {
 	int timescale = 600;
 	/* Encode the 29.97, 23.976, 59.94 framerates */
-	if(frame_rate - (int)frame_rate != 0) 
+	if(frame_rate - (int)frame_rate != 0)
 		timescale = (int)(frame_rate * 1001 + 0.5);
 	else
-		if((600 / frame_rate) - (int)(600 / frame_rate) != 0) 
+		if((600 / frame_rate) - (int)(600 / frame_rate) != 0)
 			timescale = (int)(frame_rate * 100 + 0.5);
 	return timescale;
 }
+#endif
 
-
-/* print list of things. Shamelessly stolen from export_ffmpeg.c */ 
-static int list(char *list_type) 
+/* print list of things. Shamelessly stolen from export_ffmpeg.c */
+static int list(char *list_type)
 {
     int cod = 0;
     int i = 0;
-    
+
     lqt_codec_info_t ** qi = NULL;
 
 
@@ -132,27 +137,26 @@ static int list(char *list_type)
     else {
         qi = lqt_query_registry(1, 1, 1, 0);
     }
-    
-    fprintf(stderr, "[%s] List of supported %s:\n"
-                    "[%s] Name                    comments\n"
-                    "[%s] ---------------         ---------------------------"
-                    "--------\n", MOD_NAME, list_type, MOD_NAME, MOD_NAME);
+
+    tc_log_info(MOD_NAME, "List of supported %s:", list_type);
+    tc_log_info(MOD_NAME, "Name                    comments");
+    tc_log_info(MOD_NAME, "---------------         "
+		          "-----------------------------------");
     while(qi[cod] != NULL)
-        {       
+        {
         if (list_type == QT_LIST_PARM) {
-            fprintf(stderr, "[%s]\n[%s] %s:\n", MOD_NAME, MOD_NAME,
-                qi[cod]->name);
+            tc_log_info(MOD_NAME, "%s:", qi[cod]->name);
             for(i = 0; i < qi[cod]->num_encoding_parameters; i++) {
                 if (qi[cod]->encoding_parameters[i].type != LQT_PARAMETER_SECTION) {
-                    fprintf(stderr, "[%s]  %-23s %s\n", MOD_NAME,
-                        qi[cod]->encoding_parameters[i].name, 
+                    tc_log_info(MOD_NAME, " %-23s %s",
+                        qi[cod]->encoding_parameters[i].name,
                         qi[cod]->encoding_parameters[i].real_name);
                 }
             }
         }
         else {
-            fprintf(stderr, "[%s] %-23s %s\n", MOD_NAME,
-                qi[cod]->name, 
+            tc_log_info(MOD_NAME, "%-23s %s",
+                qi[cod]->name,
                 qi[cod]->description);
         }
         cod++;
@@ -161,23 +165,7 @@ static int list(char *list_type)
 return 1;
 }
 
-/* stolen from vid_aux.c */
-/* due to a name clash between libvo and lqt, vid_aux can't be used */
-void qt_uyvytoyuy2(unsigned char *input, unsigned char *output, int width, int height)
-{
-  int i;
-  
-  for (i=0; i<width*height*2; i+=4) {
-      /* packed YUV 4:2:2 is Y[i] U[i] Y[i+1] V[i] (YUY2)*/
-      /* packed YUV 4:2:2 is U[i] Y[i] V[i] Y[i+1] (UYVY)*/
-      output[i] = input[i+1];
-      output[i+1] = input[i];
-      output[i+2] = input[i+3];
-      output[i+3] = input[i+2];
-  }
-}
-
-/* ------------------------------------------------------------ 
+/* ------------------------------------------------------------
  *
  * open outputfile
  *
@@ -185,16 +173,16 @@ void qt_uyvytoyuy2(unsigned char *input, unsigned char *output, int width, int h
 
 MOD_open
 {
-  if(param->flag == TC_VIDEO) 
+  if(param->flag == TC_VIDEO)
     return(0);
-  
-  if(param->flag == TC_AUDIO) 
+
+  if(param->flag == TC_AUDIO)
     return(0);
-  
+
   return(TC_EXPORT_ERROR);
 }
 
-/* ------------------------------------------------------------ 
+/* ------------------------------------------------------------
  *
  * init codec
  *
@@ -205,10 +193,10 @@ MOD_init
 
   int list_was_called = 0;
 
-  /* for codec parameters */  
+  /* for codec parameters */
   int jpeg_quality = 0;
   int div3_bitrate_tolerance = 500000;
-  int vorbis_max_bitrate = 192;        
+  int vorbis_max_bitrate = 192;
   int vorbis_min_bitrate = 128;
 
   /* overwriting empty parameters now saves trouble later */
@@ -220,13 +208,13 @@ MOD_init
   if (!strcasecmp(vob->ex_a_fcc, "list")) list_was_called = list(QT_LIST_AUDIO);
   if (!strcasecmp(vob->ex_profile_name, "list")) {
     int i;
-    
+
     list_was_called = list(QT_LIST_PARM);
 
     /* list special paramters at the end */
     for(i = 0; qt_param_list[i].name != NULL; i++) {
-        fprintf(stderr, "[%s]  %-23s %s\n", MOD_NAME,
-            qt_param_list[i].name, 
+        tc_log_info(MOD_NAME, "  %-23s %s",
+            qt_param_list[i].name,
             qt_param_list[i].comments);
     }
   }
@@ -247,12 +235,12 @@ MOD_init
     h = vob->ex_v_height;
 
     /* fetch qt codec from -F switch */
-    qt_codec =  strdup(vob->ex_v_fcc);
-    
+    qt_codec =  tc_strdup(vob->ex_v_fcc);
+
     /* open target file for writing */
-    if(NULL == (qtfile = quicktime_open(vob->video_out_file, 0, 1)) ){
-        fprintf(stderr,"[%s] error opening qt file '%s'\n",
-            MOD_NAME,vob->video_out_file);
+    if(NULL == (qtfile = quicktime_open((char *)vob->video_out_file, 0, 1)) ){
+        tc_log_warn(MOD_NAME, "error opening qt file '%s'",
+            vob->video_out_file);
         return(TC_EXPORT_ERROR);
     }
 
@@ -260,55 +248,62 @@ MOD_init
     /* not needed for passthrough*/
     if (vob->im_v_codec != CODEC_RAW && vob->im_v_codec != CODEC_RAW_YUV && vob->im_v_codec != CODEC_RAW_RGB) {
         if(qt_codec == NULL || strlen(qt_codec)==0) {
-            /* default */     
+            /* default */
             qt_codec = "mjpa";
             if (verbose_flag != TC_QUIET) {
-                fprintf(stderr, "[%s] INFO: Empty qt codec name. Switching to %s use '-F list'\n"
-                                "[%s]       to get a list of supported codec. \n", 
-                    MOD_NAME, qt_codec, MOD_NAME);
+                tc_log_info(MOD_NAME, "Empty qt codec name. Switching to %s use '-F list'"
+                                " to get a list of supported codec.", qt_codec);
             }
         }
 
         /* check if we can encode with this codec */
         qt_codec_info = lqt_find_video_codec_by_name(qt_codec);
         if (!qt_codec_info) {
-        fprintf(stderr,"[%s] qt video codec '%s' not supported!\n",
-            MOD_NAME,qt_codec);
+        tc_log_warn(MOD_NAME, "qt video codec '%s' not supported!", qt_codec);
             return(TC_EXPORT_ERROR);
         }
 
+#if !defined(LIBQUICKTIME_000904)
         /* set proposed video codec */
-        lqt_add_video_track(qtfile, w, h,
+        lqt_set_video(qtfile, 1, w, h, vob->ex_fps,qt_codec_info[0]);
+#else
+        /* set proposed video codec */
+        lqt_set_video(qtfile, 1, w, h,
 		tc_quicktime_get_timescale(vob->ex_fps) / vob->ex_fps+0.5,
 		tc_quicktime_get_timescale(vob->ex_fps), qt_codec_info[0]);
+#endif
     }
-    
+
     /* set color model */
     switch(vob->im_v_codec) {
         case CODEC_RGB:
             qt_cm = BC_RGB888;
-            break;
-              
-        case CODEC_YUV:
-            qt_cm = BC_YUV420P;
+            tc_cm = IMG_RGB_DEFAULT;
             break;
 
-        case CODEC_YUV422:    
+        case CODEC_YUV:
+            qt_cm = BC_YUV420P;
+            tc_cm = IMG_YUV_DEFAULT;
+            break;
+
+        case CODEC_YUV422:
+            tc_cm = IMG_YUV422P;
             qt_cm = BC_YUV422P;
             break;
 
         case CODEC_YUY2:
+            tc_cm = IMG_YUY2;
             qt_cm = BC_YUV422;
             break;
- 
+
          /* passthrough */
         case CODEC_RAW_RGB:
         case CODEC_RAW_YUV:
         case CODEC_RAW:
-            /* set out output codec to input codec */ 
+            /* set out output codec to input codec */
             if(qt_codec == NULL || strlen(qt_codec)==0) {
-                switch (vob->codec_flag) {
-                    case TC_CODEC_MJPG:
+                switch (vob->v_codec_flag) {
+                    case TC_CODEC_MJPEG:
                         quicktime_set_video(qtfile, 1, w, h, vob->ex_fps,"jpeg");
                         break;
 
@@ -328,7 +323,7 @@ MOD_init
                         quicktime_set_video(qtfile, 1, w, h, vob->ex_fps,"SVQ3");
                         break;
 
-                    case TC_CODEC_YV12:
+                    case TC_CODEC_YUV420P:
                         quicktime_set_video(qtfile, 1, w, h, vob->ex_fps,"yv12");
                         break;
 
@@ -347,70 +342,69 @@ MOD_init
                         break;
 
                     default:
-                        fprintf(stderr, "[%s] ERROR: codec '%lx' not supported for pass-through\n"
-                                        "[%s]        If you really know what you are doing you can force \n"
-                                        "[%s]        a codec via -F <vc>, '-F list' returns a list\n", 
-                            MOD_NAME, vob->codec_flag, MOD_NAME, MOD_NAME);
+                        tc_log_warn(MOD_NAME, "codec '%lx' not supported for pass-through",
+					vob->v_codec_flag);
+			tc_log_warn(MOD_NAME, "        If you really know what you are doing you can force");
+                        tc_log_warn(MOD_NAME, "        a codec via -F <vc>, '-F list' returns a list");
                         return(TC_EXPORT_ERROR);
                         break;
                 }
             }
             else {
-                fprintf(stderr,"[%s] WARNING: Overriding the output codec is almost never a good idea\n",
-                    MOD_NAME);
+                tc_log_warn(MOD_NAME,"Overriding the output codec is almost never a good idea");
                 quicktime_set_video(qtfile, 1, w, h, vob->ex_fps,qt_codec);
             }
 
-            /*fprintf(stderr," using vid\n"); */
             rawVideo = 1;
             break;
-              
+
         default:
             /* unsupported internal format */
-            fprintf(stderr,"[%s] unsupported internal video format %x\n",
-                MOD_NAME,vob->ex_v_codec);
+	    tc_log_warn(MOD_NAME, "unsupported internal video format %x",
+                vob->ex_v_codec);
             return(TC_EXPORT_ERROR);
             break;
     }
     
-    /* store tc and lqt colormodel */
-    tc_cm = vob->im_v_codec;
-    
     /* if cs conversation not supported for codec do conversation */
     /* not required for pass through */
     if (vob->im_v_codec != CODEC_RAW && vob->im_v_codec != CODEC_RAW_YUV && vob->im_v_codec != CODEC_RAW_RGB) {
         if (quicktime_writes_cmodel(qtfile, qt_cm, 0) != 1) { 
             if (verbose_flag != TC_QUIET) {
-                fprintf(stderr,"[%s] INFO: Colorspace not supported for this codec\n"
-                               "[%s]       converting to RGB\n", 
-                        MOD_NAME, MOD_NAME);
+                tc_log_info(MOD_NAME,"Colorspace not supported for this codec converting to RGB");
             }
             
             qt_cm = BC_RGB888;
             lqt_set_cmodel(qtfile, 0, qt_cm);
-            yuv2rgb_init(24, MODE_BGR);
+	    
+            tcvhandle = tcv_init();
+            if (!tcvhandle) {
+                tc_log_warn(MOD_NAME, "image conversion init failed");
+                return TC_EXPORT_ERROR;
+            }
+            
         } else {
             lqt_set_cmodel(qtfile, 0, qt_cm);
         }
     }
-    
+
     /* set codec parameters */
     /* tc uses kb */
-    divx_bitrate = vob->divxbitrate * 1000; 
-    /* if max bitrate > bitrate  use difference for bitrate tolerance */        
+    divx_bitrate = vob->divxbitrate * 1000;
+    /* if max bitrate > bitrate  use difference for bitrate tolerance */
     if (vob->video_max_bitrate > vob->divxbitrate) div3_bitrate_tolerance = (vob->video_max_bitrate - vob->divxbitrate) * 1000;
 
-    
-    /* Audio */    
+
+    /* Audio */
     /* must be changed when it's changed in lqt; "bit_rate" conflicts with ffmpeg video codec */
     if (strcmp(qt_codec,"ffmpeg_mp2") == 0||
         strcmp(qt_codec,"ffmpeg_mp3") == 0||
-        strcmp(qt_codec,"ffmpeg_ac3") == 0) 
+        strcmp(qt_codec,"ffmpeg_ac3") == 0)
         quicktime_set_parameter(qtfile, "bit_rate", &vob->mp3bitrate);
-    
+
     if (strcmp(qt_codec,"lame") == 0)
         quicktime_set_parameter(qtfile, "mp3_bitrate", &vob->mp3bitrate);
-    
+
     /* have to check this */
     if (strcmp(qt_codec,"vorbis") == 0) {
         quicktime_set_parameter(qtfile, "vorbis_bitrate", &vob->mp3bitrate);
@@ -418,62 +412,61 @@ MOD_init
         quicktime_set_parameter(qtfile, "vorbis_min_bitrate", &vorbis_min_bitrate);
         quicktime_set_parameter(qtfile, "vorbis_vbr", &vob->a_vbr);
     }
-    
-    /* Video */    
+
+    /* Video */
     /* jpeg_quality == 0-100 ; tc quality setting (-Q) ==  1-5 */
     jpeg_quality = 20 * vob->divxquality;
-    
+
     if (strcmp(qt_codec,"mjpa") == 0||
         strcmp(qt_codec,"jpeg") == 0)
         quicktime_set_parameter(qtfile, "jpeg_quality", &jpeg_quality);
-    
-    /* looks terrible :/ */ 
-    if (strcmp(qt_codec,"ffmpeg_mjpg") == 0|| 
+
+    /* looks terrible :/ */
+    if (strcmp(qt_codec,"ffmpeg_mjpg") == 0||
         strcmp(qt_codec,"ffmpeg_h263p") == 0||
         strcmp(qt_codec,"ffmpeg_h263") == 0||
         strcmp(qt_codec,"ffmpeg_msmpeg4v3") == 0||
-        strcmp(qt_codec,"ffmpeg_msmpeg4v2") == 0|| 
-        strcmp(qt_codec,"ffmpeg_msmpeg4v1") == 0|| 
+        strcmp(qt_codec,"ffmpeg_msmpeg4v2") == 0||
+        strcmp(qt_codec,"ffmpeg_msmpeg4v1") == 0||
         strcmp(qt_codec,"ffmpeg_msmpg1") == 0||
         strcmp(qt_codec,"ffmpeg_mpg4") == 0){
-    
+
         int min_bitrate = 0;
-    
+
         quicktime_set_parameter(qtfile, "flags_gray", &vob->decolor); /* set format different for this */
-      
+
         switch (vob->decolor) {
         case 1:
             quicktime_set_parameter(qtfile, "aspect_ratio_info", "Square");
             break;
-      
+
         case 2:
             quicktime_set_parameter(qtfile, "aspect_ratio_info", "4:3");
             break;
-            
+
         case 3:
             quicktime_set_parameter(qtfile, "aspect_ratio_info", "16:9");
             break;
-      
+
         default:
-            fprintf(stderr, "[%s] WARNING: Given aspect ratio not supported, using default \n",
-                MOD_NAME);
+            tc_log_warn(MOD_NAME, "Given aspect ratio not supported, using default");
             break;
         }
-    
-        quicktime_set_parameter(qtfile, "flags_gray", &vob->decolor);  
+
+        quicktime_set_parameter(qtfile, "flags_gray", &vob->decolor);
         quicktime_set_parameter(qtfile, "bit_rate", &vob->divxbitrate);
         quicktime_set_parameter(qtfile, "bit_rate_tolerance", &div3_bitrate_tolerance);
 
         min_bitrate = vob->divxbitrate - div3_bitrate_tolerance;
-        quicktime_set_parameter(qtfile, "rc_max_rate", &vob->video_max_bitrate);        
+        quicktime_set_parameter(qtfile, "rc_max_rate", &vob->video_max_bitrate);
         quicktime_set_parameter(qtfile, "qmax", &vob->max_quantizer);
         quicktime_set_parameter(qtfile, "qmin", &vob->min_quantizer);
 
-      
+
         if (!strcmp(qt_codec,"ffmpeg_mjpg"))
             quicktime_set_parameter(qtfile, "gob_size", &vob->divxkeyframes);
     }
-    
+
     if (strcmp(vob->ex_v_fcc,"opendivx") == 0) {
         quicktime_set_parameter(qtfile, "divx_bitrate", &divx_bitrate);
         quicktime_set_parameter(qtfile, "divx_rc_period", &vob->rc_period);
@@ -485,81 +478,66 @@ MOD_init
         quicktime_set_parameter(qtfile, "divx_quantizer", &vob->min_quantizer);
         quicktime_set_parameter(qtfile, "divx_quality", &vob->quality);
     }
-    
+
     if (strcmp(qt_codec,"rtjpeg") == 0)
         quicktime_set_parameter(qtfile, "rtjpeg_quality", &jpeg_quality);
 
 
     /* set extended parameters */
-    if (vob->ex_profile_name != NULL) {  /* check for extended option */
-        char param[strlen(vob->ex_profile_name)], 
-             qtvalue[strlen(vob->ex_profile_name)]; 
-            
-        int k = 0;
-        int i, j = 0;
-        int qtvalue_i = 0; /* for int values */ 
-            
-        for(i=0;i<=strlen(vob->ex_profile_name);i++,j++) {
-            /* try to catch bad input */
-            if (vob->ex_profile_name[i] == ','){
-                fprintf(stderr, "[%s] bad -F option found \n"
-                                "[%s] try something like this: \"-F vc,ac,opt1=val,opt2=val...\"\n",
-                    MOD_NAME, MOD_NAME);
-                return(TC_EXPORT_ERROR);
-                break;
-            }
-            if (vob->ex_profile_name[i] == '=') {
-                param[j] = (char)NULL;
-                j=-1; /* set to 0 by for loop above */
-                
-                for(i++,k=0;i<=strlen(vob->ex_profile_name) && vob->ex_profile_name[i] != (char)NULL;i++,k++) {
-                    /* try to catch bad input */        
-                    if (vob->ex_profile_name[i] == '=') {
-                        fprintf(stderr, "[%s] bad -F option found, aborting ...\n"
-                                        "[%s] try something like this: \"-F vc,ac,opt1=val,opt2=val...\"\n",
-                            MOD_NAME, MOD_NAME);
-                        return(TC_EXPORT_ERROR);
-                        break;
-                    }
-                    if (vob->ex_profile_name[i] != ',') qtvalue[k] = vob->ex_profile_name[i];
-                    else break;
-                }
+    if (vob->ex_profile_name != NULL) {
+        int i;
+        char meta[128];
 
-                qtvalue[k] = (char)NULL;
-                /* exception for copyright, name, info */
-                /* everything else is assumed to be of type int */
-                if (strcmp(param, "copyright") == 0||
-                    strcmp(param, "name") == 0||
-                    strcmp(param, "info") == 0) { 
-                    if (strcmp(param, "name") == 0) quicktime_set_name(qtfile, qtvalue);
-                    if (strcmp(param, "copyright") == 0) quicktime_set_copyright(qtfile, qtvalue);                    
-                    if (strcmp(param, "info") == 0) quicktime_set_info(qtfile, qtvalue);
+        if (optstr_get(vob->ex_profile_name, "copyright", "%128[^:]", meta) > 0) {
+            quicktime_set_copyright(qtfile, meta);
+        }
+        if(optstr_get(vob->ex_profile_name, "name", "%128[^:]", meta) > 0) {
+            quicktime_set_name(qtfile, meta);
+        }
+        if(optstr_get(vob->ex_profile_name, "info", "%128[^:]", meta) > 0) {
+            quicktime_set_name(qtfile, meta);
+        }
+
+        for (i = 0; i < (* qt_codec_info)->num_encoding_parameters; i++) {
+            lqt_parameter_info_t param = (* qt_codec_info)->encoding_parameters[i];
+
+            if (param.type == LQT_PARAMETER_INT) {
+                int val;
+                if (optstr_get(vob->ex_profile_name, param.name, "%i", &val) > 0) {
+                    lqt_set_video_parameter(qtfile, 0, param.name, &val);
                 }
-                else {
-                    qtvalue_i = atoi(qtvalue);
-                    quicktime_set_parameter(qtfile, param, &qtvalue_i);                 
+            }
+            else if (param.type == LQT_PARAMETER_FLOAT) {
+                float val;
+                if (optstr_get(vob->ex_profile_name, param.name, "%f", &val) > 0) {
+                    lqt_set_video_parameter(qtfile, 0, param.name, &val);
+                }
+            }
+            else if (param.type == LQT_PARAMETER_STRING || param.type == LQT_PARAMETER_STRINGLIST) {
+                char val[128];
+                if (optstr_get(vob->ex_profile_name, param.name, "%128[^:]", val) > 0) {
+                    lqt_set_video_parameter(qtfile, 0, param.name, val);
                 }
             }
-            else param[j] = vob->ex_profile_name[i];
         }
     }
 
     /* alloc row pointers for frame encoding */
-    row_ptr = malloc (sizeof(char *) * h);        
+    row_ptr = malloc (sizeof(char *) * h);
 
-    /* allocate tmp buffer cs conversation */
-    tmp_buf = malloc (w * h * 3);
+    /* same for temp buffer ... used during yuy2/yv12 encoding */
+    tmp_buf = malloc (w*h*2);
 
     /* verbose */
-    fprintf(stderr,"[%s] video codec='%s' w=%d h=%d fps=%g\n",
-	    MOD_NAME,qt_codec,w,h,vob->ex_fps);
+    tc_log_info(MOD_NAME, "video codec='%s' w=%d h=%d fps=%g",
+	    qt_codec,w,h,vob->ex_fps);
 
     return(0);
   }
 
   /* audio setup -------------------------------------------------- */
   if(param->flag == TC_AUDIO){
-    char *qt_codec;
+    const char *qt_codec;
     lqt_codec_info_t ** qt_codec_info = 0;
 
     /* no audio setup if we don't have any channels (size == 0 might be better?)*/
@@ -567,8 +545,7 @@ MOD_init
 
     /* check given audio format */
     if((vob->dm_chan!=1)&&(vob->dm_chan!=2)) {
-        fprintf(stderr, "[%s] Only mono or stereo audio supported\n",
-            MOD_NAME);
+        tc_log_warn(MOD_NAME, "Only mono or stereo audio supported");
         return(TC_EXPORT_ERROR);
     }
     channels = vob->dm_chan;
@@ -579,13 +556,13 @@ MOD_init
     if(qt_codec == NULL || strlen(qt_codec)==0) {
         qt_codec = "ima4";
         if (verbose_flag != TC_QUIET) {
-            fprintf(stderr, "[%s] INFO: empty qt codec name - switching to %s use '-F ,list'\n"
-                            "[%s]       to get a list of supported codec \n", 
-                MOD_NAME, qt_codec, MOD_NAME);
+            tc_log_info(MOD_NAME, "empty qt codec name - switching to %s"
+			          " use '-F ,list'", qt_codec);
+	    tc_log_info(MOD_NAME, "to get a list of supported codec");
         }
     }
-     
-     
+
+
     /* check encoder mode */
     switch(vob->im_a_codec) {
         case CODEC_PCM:
@@ -599,16 +576,16 @@ MOD_init
 
         default:
             /* unsupported internal format */
-            fprintf(stderr,"[%s] unsupported internal audio format %x\n",
-                MOD_NAME,vob->ex_v_codec);
+            tc_log_warn(MOD_NAME, "unsupported internal audio format %x",
+                vob->ex_v_codec);
             return(TC_EXPORT_ERROR);
             break;
     }
 
     qt_codec_info = lqt_find_audio_codec_by_name(qt_codec);
     if (!qt_codec_info){
-        fprintf(stderr,"[%s] qt audio codec '%s' unsupported\n",
-            MOD_NAME,qt_codec);
+        tc_log_warn(MOD_NAME, "qt audio codec '%s' unsupported",
+            qt_codec);
         return(TC_EXPORT_ERROR);
     }
 
@@ -616,15 +593,15 @@ MOD_init
     lqt_set_audio(qtfile,channels,vob->a_rate,bits,qt_codec_info[0]);
 
     /* verbose */
-    fprintf(stderr,"[%s] audio codec='%s' bits=%d chan=%d rate=%d\n",
-	    MOD_NAME,qt_codec,bits,channels,vob->a_rate);
+    tc_log_info(MOD_NAME, "audio codec='%s' bits=%d chan=%d rate=%d",
+	    qt_codec,bits,channels,vob->a_rate);
     return(0);
   }
-  
+
   return(TC_EXPORT_ERROR);
 }
 
-/* ------------------------------------------------------------ 
+/* ------------------------------------------------------------
  *
  * encode and export frame
  *
@@ -635,82 +612,82 @@ MOD_encode
 {
   /* video -------------------------------------------------------- */
   if(param->flag == TC_VIDEO) {
-    unsigned char *ptr = (unsigned char *)param->buffer;
+    vob_t *vob = tc_get_vob();
     
     /* raw mode is easy */
     if(rawVideo) {
         /* add divx keyframes if needed */
-        if(quicktime_divx_is_key(ptr, (unsigned char)param->size) == 1)
-            quicktime_insert_keyframe(qtfile, (int)tc_get_frames_encoded, 0);
-        if(quicktime_write_frame(qtfile, ptr, param->size,0)<0) {
-            fprintf(stderr, "[%s] error writing raw video frame\n",
-                MOD_NAME);
+        if(quicktime_divx_is_key(param->buffer, param->size) == 1) quicktime_insert_keyframe(qtfile, (int)tc_get_frames_encoded(), 0);
+        if(quicktime_write_frame(qtfile,param->buffer,param->size,0)<0) {
+            tc_log_warn(MOD_NAME, "error writing raw video frame");
             return(TC_EXPORT_ERROR);
         }
     }
 
     /* encode frame */
     else {
-        int iy,sl;
-	
+        uint8_t *ptr = param->buffer;
+        int iy;
+
         switch(qt_cm) {
             case BC_RGB888:
-                if (tc_cm == CODEC_YUV) {
-                    yuv2rgb((void *)tmp_buf,
-                        ptr, ptr + w*h, ptr + (w*h*5)/4,
-                        w, h, w*3, w, w/2);
-                    ptr = tmp_buf;
+                /* convert to rgb if unsupported */
+                if (tc_cm != IMG_RGB24) {
+                    if (!tcv_convert(tcvhandle, param->buffer, param->buffer,
+                                     vob->ex_v_width, vob->ex_v_height,
+                                     tc_cm, IMG_RGB24)) {
+                        tc_log_warn(MOD_NAME, "image format conversion failed");
+                        return TC_EXPORT_ERROR;
+                    }
                 }
-		
-                /* setup row pointers for RGB: inverse! */
-                sl = w*3;
-                for(iy=0;iy<h;iy++){
+
+                /* setup row pointers for RGB */
+                for (iy = 0; iy < h; iy++) {
                     row_ptr[iy] = ptr;
-                    ptr += sl;
+                    ptr += w*3;
                 }
                 break;
-
             case BC_YUV420P:
-                /* setup row pointers for YUV420P: inverse! */
+                /* setup row pointers for YUV420P */
                 row_ptr[0] = ptr;
                 ptr = ptr + (h * w);
+                /* note, quicktime wants YV12 so we reverse the planes */
                 row_ptr[2] = ptr;
                 ptr = ptr + (h * w) / 4;
                 row_ptr[1] = ptr;
                 break;
-
-            case CODEC_YUY2:
+            case BC_YUV422P:
+                /* setup row pointers for YUV422P */
+                row_ptr[0] = ptr;
+                ptr = ptr + (h * w);
+                row_ptr[1] = ptr;
+                ptr = ptr + (h * w) / 2;
+                row_ptr[2] = ptr;
+                break;
             case BC_YUV422:
-                /* setup row pointers for YUV422: inverse ?*/
-                sl = w*2;                        
-                if (qt_cm != CODEC_YUY2) {
-                    /* convert uyvy to yuy2 */   /* find out if lqt supports uyvy byteorder */ 
-                    qt_uyvytoyuy2(ptr, tmp_buf, w, h);
-                    ptr = tmp_buf;
-                }
-                for(iy=0;iy<h;iy++){
+                /* setup row pointers for YUY2 */
+                for (iy = 0; iy < h; iy++) {
                     row_ptr[iy] = ptr;
-                    ptr += sl;
+                    ptr += w*2;
                 }
                 break;
         }
 
         if(quicktime_encode_video(qtfile, row_ptr, 0)<0) {
-            fprintf(stderr, "[%s] error encoding video frame\n",MOD_NAME);
+            tc_log_warn(MOD_NAME, "error encoding video frame");
             return(TC_EXPORT_ERROR);
         }
     }
 
     return(0);
-  }  
+  }
   /* audio -------------------------------------------------------- */
   if(param->flag == TC_AUDIO){
     /* raw mode is easy */
     if(rawAudio) {
         if(quicktime_write_frame(qtfile,
-            (unsigned char *)param->buffer,param->size,0)<0) {
-            fprintf(stderr, "[%s] error writing raw audio frame\n",
-                MOD_NAME);
+            param->buffer,param->size,0)<0) {
+            tc_log_warn(MOD_NAME, "error writing raw audio frame");
             return(TC_EXPORT_ERROR);
         }
     }
@@ -734,9 +711,9 @@ MOD_encode
         if(bits==8) {
             /* UNTESTED: please verify :) */
             if(channels==1) {
-                for(s=0;s<samples;s++) 
+                for(s=0;s<samples;s++)
                     audbuf0[s] = ((((int16_t)param->buffer[s]) << 8)-0x8000);
-            } 
+            }
             else /* stereo */ {
                 for(s=0,t=0;s<samples;s++,t+=2) {
                     audbuf0[s] = ((((int16_t)param->buffer[t]) << 8)-0x8000);
@@ -747,7 +724,7 @@ MOD_encode
         else /* 16 bit */ {
             if(channels==1) {
                 aptr[0] = (int16_t *)param->buffer;
-            } 
+            }
             else /* stereo */ {
                 /* decouple channels */
                 for(s=0,t=0;s<samples;s++,t+=2) {
@@ -756,39 +733,37 @@ MOD_encode
                 }
             }
         }
-        
+
         /* encode audio samples */
         if ((quicktime_encode_audio(qtfile, aptr, NULL, samples)<0)){
-            fprintf(stderr,"[%s] error encoding audio frame\n",MOD_NAME);
+            tc_log_warn(MOD_NAME, "error encoding audio frame");
             return(TC_EXPORT_ERROR);
         }
     }
-        
+
     return(0);
   }
-  
+
   return(TC_EXPORT_ERROR);
 }
 
 
-/* ------------------------------------------------------------ 
+/* ------------------------------------------------------------
  *
  * stop encoder
  *
  * ------------------------------------------------------------*/
-  
+
 MOD_stop
-{  
-  
+{
+
   if(param->flag == TC_VIDEO) {
     /* free row pointers */
     if(row_ptr!=NULL)
         free(row_ptr);
-    if(tmp_buf!=NULL)
-        free(tmp_buf);
     return(0);
   }
-  
+
   if(param->flag == TC_AUDIO) {
     /* free audio buffers */
     if(audbuf0!=NULL)
@@ -797,29 +772,29 @@ MOD_stop
         free(audbuf1);
     return(0);
   }
-  
+
   return(TC_EXPORT_ERROR);
 }
 
-/* ------------------------------------------------------------ 
+/* ------------------------------------------------------------
  *
  * close outputfile
  *
  * ------------------------------------------------------------*/
 
 MOD_close
-{  
+{
   if(param->flag == TC_VIDEO){
     /* close quicktime file */
     quicktime_close(qtfile);
     qtfile=NULL;
     return(0);
     }
-  
+
   if(param->flag == TC_AUDIO) {
     return(0);
   }
-  
+
   return(TC_EXPORT_ERROR);
-  
+
 }
